@@ -36,7 +36,10 @@ static GtkBuilder *builder;
 static ZenityData *zen_data;
 static GIOChannel *channel;
 
+static gint pulsate_timeout = -1;
 static gboolean autokill;
+static gboolean no_cancel;
+static gboolean auto_close;
 
 gint zenity_progress_timeout (gpointer data);
 gint zenity_progress_pulsate_timeout (gpointer data);
@@ -50,6 +53,25 @@ zenity_progress_pulsate_progress_bar (gpointer user_data)
   return TRUE;
 }
 
+static void
+zenity_progress_pulsate_stop ()
+{
+  if (pulsate_timeout > 0) {
+    g_source_remove (pulsate_timeout);
+    pulsate_timeout = -1;
+  }
+}
+
+static void
+zenity_progress_pulsate_start (GObject *progress_bar)
+{
+  if (pulsate_timeout == -1) {
+    pulsate_timeout = g_timeout_add (100,
+                                     zenity_progress_pulsate_progress_bar,
+                                     progress_bar);
+  }
+}
+
 static gboolean
 zenity_progress_handle_stdin (GIOChannel   *channel,
                               GIOCondition  condition,
@@ -58,7 +80,6 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
   static ZenityProgressData *progress_data;
   static GObject *progress_bar;
   static GObject *progress_label;
-  static gint pulsate_timeout = -1;
   float percentage = 0.0;
   
   progress_data = (ZenityProgressData *) data;
@@ -72,8 +93,7 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
     string = g_string_new (NULL);
 
     if (progress_data->pulsate) {
-      if (pulsate_timeout == -1)
-        pulsate_timeout = g_timeout_add (100, zenity_progress_pulsate_progress_bar, progress_bar);
+      zenity_progress_pulsate_start (progress_bar);
     }
 
     while (channel->is_readable != TRUE)
@@ -105,26 +125,58 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
         match = g_strstr_len (string->str, strlen (string->str), "#");
         match++;
         gtk_label_set_text (GTK_LABEL (progress_label), g_strcompress(g_strchomp (g_strchug (match))));
+
+      } else if (g_str_has_prefix (string->str, "pulsate")) {
+        gchar *colon, *command, *value;
+
+        zenity_util_strip_newline (string->str);
+
+        colon = strchr(string->str, ':');
+        if (colon == NULL) {
+            continue;
+        }
+
+        /* split off the command and value */
+        command = g_strstrip (g_strndup (string->str, colon - string->str));
+
+        value = colon + 1;
+        while (*value && g_ascii_isspace (*value)) value++;
+
+        if (!g_ascii_strcasecmp (value, "false")) {
+          zenity_progress_pulsate_stop ();
+
+          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar),
+                                         progress_data->percentage / 100.0);
+        } else {
+          zenity_progress_pulsate_start (progress_bar);
+        }
+
+        g_free (command);
       } else {
 
         if (!g_ascii_isdigit (*(string->str)))
           continue;
 
         /* Now try to convert the thing to a number */
-        percentage = atoi (string->str);
-        if (percentage >= 100) {
-	  GObject *button;
-	  button = gtk_builder_get_object(builder, "zenity_progress_ok_button");
-          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 1.0);
-	  gtk_widget_set_sensitive(GTK_WIDGET (button), TRUE);
-	  gtk_widget_grab_focus(GTK_WIDGET (button));
-	  if (progress_data->autoclose) {
-		zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
-		gtk_main_quit();
-		
-	  }
-	} else
-          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), percentage / 100.0);
+        percentage = CLAMP(atoi (string->str), 0, 100);
+
+        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar),
+                                       percentage / 100.0);
+
+        progress_data->percentage = percentage;
+
+        if (percentage == 100) {
+          GObject *button;
+
+          button = gtk_builder_get_object(builder, "zenity_progress_ok_button");
+          gtk_widget_set_sensitive(GTK_WIDGET (button), TRUE);
+          gtk_widget_grab_focus(GTK_WIDGET (button));
+
+          if (progress_data->autoclose) {
+            zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
+            gtk_main_quit();
+          }
+        }
       }
 
     } while (g_io_channel_get_buffer_condition (channel) == G_IO_IN);
@@ -136,20 +188,18 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
     GtkWidget *button;
 
     button = GTK_WIDGET (gtk_builder_get_object (builder,
-    						 "zenity_progress_ok_button"));
+                                                 "zenity_progress_ok_button"));
     gtk_widget_set_sensitive (button, TRUE);
     gtk_widget_grab_focus (button);
 
     button = GTK_WIDGET (gtk_builder_get_object (builder,
-    					      "zenity_progress_cancel_button"));
+                         "zenity_progress_cancel_button"));
+
     gtk_widget_set_sensitive (button, FALSE);
-		
+
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 1.0);
 
-    if (progress_data->pulsate) {
-      g_source_remove (pulsate_timeout);
-      pulsate_timeout = -1;
-    }
+    zenity_progress_pulsate_stop ();
 
     g_object_unref (builder);
 
@@ -157,7 +207,7 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
       gtk_main_quit();
     }
-		
+
     g_io_channel_shutdown (channel, TRUE, NULL);
     return FALSE;
   }
@@ -179,6 +229,7 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
   GtkWidget *dialog;
   GObject *text;
   GObject *progress_bar;
+  GObject *cancel_button,*ok_button;
 
   zen_data = data;
   builder = zenity_util_load_ui_file ("zenity_progress_dialog", NULL);
@@ -187,15 +238,15 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
     data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
     return;
   }
-	
+
   gtk_builder_connect_signals (builder, NULL);
-	
+
   dialog = GTK_WIDGET (gtk_builder_get_object (builder,
-  					       "zenity_progress_dialog"));
+                                               "zenity_progress_dialog"));
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (zenity_progress_dialog_response), data);
-	
+
   if (data->dialog_title)
     gtk_window_set_title (GTK_WINDOW (dialog), data->dialog_title);
 
@@ -217,11 +268,25 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
 
   autokill = progress_data->autokill;
 
+  auto_close = progress_data->autoclose;
+  ok_button = gtk_builder_get_object (builder, "zenity_progress_ok_button");
+
+  no_cancel = progress_data->no_cancel;
+  cancel_button = gtk_builder_get_object (builder, "zenity_progress_cancel_button");
+
+  if (no_cancel) {
+     gtk_widget_hide (GTK_WIDGET(cancel_button));
+     gtk_window_set_deletable (GTK_WINDOW (dialog), FALSE);
+  }
+
+  if (no_cancel && auto_close)
+     gtk_widget_hide(GTK_WIDGET(ok_button));
+
   zenity_util_show_dialog (dialog);
   zenity_progress_read_info (progress_data);
 
   if(data->timeout_delay > 0) {
-    g_timeout_add (data->timeout_delay * 1000, (GSourceFunc) zenity_util_timeout_handle, NULL);
+    g_timeout_add_seconds (data->timeout_delay, (GSourceFunc) zenity_util_timeout_handle, NULL);
   }
 
   gtk_main ();
@@ -234,12 +299,12 @@ zenity_progress_dialog_response (GtkWidget *widget, int response, gpointer data)
     case GTK_RESPONSE_OK:
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
       break;
-		
+
     case GTK_RESPONSE_CANCEL:
       /* We do not want to kill the parent process, in order to give the user
          the ability to choose the action to be taken. See bug #310824.
-	 But we want to give people the option to choose this behavior.
-		 -- Monday 27, March 2006
+         But we want to give people the option to choose this behavior.
+         -- Monday 27, March 2006
       */
       if (autokill) {
         kill (getppid (), 1);
